@@ -1,8 +1,6 @@
 //
 // ICMPClientTest.h
 //
-// $Id: //poco/1.4/Net/testsuite/src/ICMPClientTest.h#1 $
-//
 // Definition of the ICMPClientTest class.
 //
 // Copyright (c) 2006, Applied Informatics Software Engineering GmbH.
@@ -29,6 +27,7 @@ public:
 	~ICMPClientTest();
 
 	void testPing();
+	void testBigPing();
 
 	void setUp();
 	void tearDown();
@@ -41,7 +40,8 @@ public:
 	void onEnd(const void* pSender, Poco::Net::ICMPEventArgs& args);
 
 private:
-	Poco::Net::ICMPClient _icmpClient;
+	void registerDelegates(const Poco::Net::ICMPClient& icmpClient);
+	void unregisterDelegates(const Poco::Net::ICMPClient& icmpClient);
 };
 
 
